Stan Math Library: stan/math/prim/scal/prob/inv_gamma_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

inv_gamma_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_INV_GAMMA_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_INV_GAMMA_CDF_HPP
3
4 #include <boost/random/gamma_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
10 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
11 #include <stan/math/prim/scal/err/check_not_nan.hpp>
12 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/gamma_q.hpp>
17 #include <stan/math/prim/scal/fun/digamma.hpp>
18 #include <stan/math/prim/scal/meta/length.hpp>
19 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
20 #include <stan/math/prim/scal/meta/VectorView.hpp>
21 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
22 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
23 #include <stan/math/prim/scal/meta/return_type.hpp>
24 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
25 #include <stan/math/prim/scal/meta/include_summand.hpp>
26 #include <cmath>
27 #include <limits>
28
29 namespace stan {
30
31 namespace math {
32
49 template <typename T_y, typename T_shape, typename T_scale>
50 typename return_type<T_y, T_shape, T_scale>::type
51 inv_gamma_cdf(const T_y& y, const T_shape& alpha, const T_scale& beta) {
52 typedef typename stan::partials_return_type<T_y, T_shape, T_scale>::type
53 T_partials_return;
54
55 // Size checks
56 if (!(stan::length(y) && stan::length(alpha) && stan::length(beta)))
57 return 1.0;
58
59 // Error checks
60 static const char* function("stan::math::inv_gamma_cdf");
61
62 using stan::math::check_positive_finite;
63 using stan::math::check_not_nan;
64 using stan::math::check_consistent_sizes;
65 using stan::math::check_greater_or_equal;
66 using stan::math::check_less_or_equal;
67 using stan::math::check_nonnegative;
68 using stan::math::value_of;
69 using boost::math::tools::promote_args;
70 using std::exp;
71
72 T_partials_return P(1.0);
73
74 check_positive_finite(function, "Shape parameter", alpha);
75 check_positive_finite(function, "Scale parameter", beta);
76 check_not_nan(function, "Random variable", y);
77 check_nonnegative(function, "Random variable", y);
78 check_consistent_sizes(function,
79 "Random variable", y,
80 "Shape parameter", alpha,
81 "Scale Parameter", beta);
82
83 // Wrap arguments in vectors
84 VectorView<const T_y> y_vec(y);
85 VectorView<const T_shape> alpha_vec(alpha);
86 VectorView<const T_scale> beta_vec(beta);
87 size_t N = max_size(y, alpha, beta);
88
89 OperandsAndPartials<T_y, T_shape, T_scale>
90 operands_and_partials(y, alpha, beta);
91
92 // Explicit return for extreme values
93 // The gradients are technically ill-defined, but treated as zero
94
95 for (size_t i = 0; i < stan::length(y); i++) {
96 if (value_of(y_vec[i]) == 0)
97 return operands_and_partials.value(0.0);
98 }
99
100 // Compute CDF and its gradients
101 using stan::math::gamma_q;
102 using stan::math::digamma;
103 using boost::math::tgamma;
104 using std::exp;
105 using std::pow;
106
107 // Cache a few expensive function calls if nu is a parameter
108 VectorBuilder<!is_constant_struct<T_shape>::value,
109 T_partials_return, T_shape>
110 gamma_vec(stan::length(alpha));
111 VectorBuilder<!is_constant_struct<T_shape>::value,
112 T_partials_return, T_shape>
113 digamma_vec(stan::length(alpha));
114
115 if (!is_constant_struct<T_shape>::value) {
116 for (size_t i = 0; i < stan::length(alpha); i++) {
117 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
118 gamma_vec[i] = tgamma(alpha_dbl);
119 digamma_vec[i] = digamma(alpha_dbl);
120 }
121 }
122
123 // Compute vectorized CDF and gradient
124 for (size_t n = 0; n < N; n++) {
125 // Explicit results for extreme values
126 // The gradients are technically ill-defined, but treated as zero
127 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity())
128 continue;
129
130 // Pull out values
131 const T_partials_return y_dbl = value_of(y_vec[n]);
132 const T_partials_return y_inv_dbl = 1.0 / y_dbl;
133 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
134 const T_partials_return beta_dbl = value_of(beta_vec[n]);
135
136 // Compute
137 const T_partials_return Pn = gamma_q(alpha_dbl, beta_dbl * y_inv_dbl);
138
139 P *= Pn;
140
141 if (!is_constant_struct<T_y>::value)
142 operands_and_partials.d_x1[n] += beta_dbl * y_inv_dbl * y_inv_dbl
143 * exp(-beta_dbl * y_inv_dbl) * pow(beta_dbl
144 * y_inv_dbl, alpha_dbl-1)
145 / tgamma(alpha_dbl) / Pn;
146 if (!is_constant_struct<T_shape>::value)
147 operands_and_partials.d_x2[n]
148 += stan::math::grad_reg_inc_gamma(alpha_dbl, beta_dbl
149 * y_inv_dbl, gamma_vec[n],
150 digamma_vec[n]) / Pn;
151 if (!is_constant_struct<T_scale>::value)
152 operands_and_partials.d_x3[n] += - y_inv_dbl
153 * exp(-beta_dbl * y_inv_dbl)
154 * pow(beta_dbl * y_inv_dbl, alpha_dbl-1)
155 / tgamma(alpha_dbl) / Pn;
156 }
157
158 if (!is_constant_struct<T_y>::value) {
159 for (size_t n = 0; n < stan::length(y); ++n)
160 operands_and_partials.d_x1[n] *= P;
161 }
162 if (!is_constant_struct<T_shape>::value) {
163 for (size_t n = 0; n < stan::length(alpha); ++n)
164 operands_and_partials.d_x2[n] *= P;
165 }
166 if (!is_constant_struct<T_scale>::value) {
167 for (size_t n = 0; n < stan::length(beta); ++n)
168 operands_and_partials.d_x3[n] *= P;
169 }
170
171 return operands_and_partials.value(P);
172 }
173 }
174 }
175
176 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::grad_reg_inc_gamma
T grad_reg_inc_gamma(T a, T z, T g, T dig, T precision=1e-6)
Definition: grad_reg_inc_gamma.hpp:16
return_type.hpp
gamma_q.hpp
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
check_less_or_equal.hpp
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_less_or_equal
bool check_less_or_equal(const char *function, const char *name, const T_y &y, const T_high &high)
Return true if y is less or equal to high.
Definition: check_less_or_equal.hpp:81
length.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::math::tgamma
fvar< T > tgamma(const fvar< T > &x)
Definition: tgamma.hpp:15
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
stan::math::gamma_q
fvar< T > gamma_q(const fvar< T > &x1, const fvar< T > &x2)
Definition: gamma_q.hpp:15
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::inv_gamma_cdf
return_type< T_y, T_shape, T_scale >::type inv_gamma_cdf(const T_y &y, const T_shape &alpha, const T_scale &beta)
The CDF of an inverse gamma density for y with the specified shape and scale parameters.
Definition: inv_gamma_cdf.hpp:51
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.